#include "graph_lib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>

#define LOG_DEBUG 1
#define LOG_WARN 2
#define LOG_ERR 3
#define LOG_LEVEL LOG_WARN


int combin[10][3] = {
	{ 0, 1, 2 },
	{ 0, 1, 3 },
	{ 0, 1, 4 },
	{ 0, 2, 3 },
	{ 0, 2, 4 },
	{ 0, 3, 4 },
	{ 1, 2, 3 },
	{ 1, 2, 4 },
	{ 1, 3, 4 },
	{ 2, 3, 4 },
};


int complement[10][2] = {
	{ 3, 4 },
	{ 2, 4 },
	{ 2, 3 },
	{ 1, 4 },
	{ 1, 3 },
	{ 1, 2 },
	{ 0, 4 },
	{ 0, 3 },
	{ 0, 2 },
	{ 0, 1 },
};

void set_prefix( coloring_t *coloring, int num_vertices_in_prefix, int num_edges, int *indexes ) {
	
	int i, j;
	
	for ( i = 0; i < num_vertices_in_prefix; i++ ) {
		for ( j = 0; j < 3; j++ ) {
			coloring->colors[3*i+j] = combin[indexes[i]][j];
		}
	}
	
	for ( i = num_vertices_in_prefix * 3; i < num_edges; i++ ) {
		coloring->colors[i] = -1;
	}
}


int is_triangle( int *string ) {
	int counts[5];
	int i, number_of_ones = 0, number_of_twos = 0;
	for ( i = 0; i < 5; i++ ) {
		counts[i]=0;	
	}
	
	for ( i = 0; i < 4; i++ ) {
		counts[	string[i] ]++;
	}
	
	for ( i = 0; i < 5; i++ ) {
		if (counts[i] == 1) number_of_ones++;
		if (counts[i] == 2) number_of_twos++;
	}

	if (number_of_ones != 2 || number_of_twos != 1) {
		return 0;
	}
	
	for ( i = 4; i < 6; i++ ) {
		counts[	string[i] ]++;
	}

	number_of_ones = 0;
	number_of_twos = 0;
	
	for ( i = 0; i < 5; i++ ) {
		if (counts[i] == 1) number_of_ones++;
		if (counts[i] == 2) number_of_twos++;
	}
	
	if (number_of_ones != 0 || number_of_twos != 3) {
		return 0;
	}
	
	return 1;
}

void normalise( int *string ) {
	int permutation[5], m;

	//translation			
	for ( m = 0; m < 5; m++ ) {
		permutation[m] = -1;
	}
	
	int current = 0;
	for ( m = 0; m < 12; m++ ) {
		if ( permutation[string[m]] == -1 ) {
			permutation[string[m]] = current;
			current++;
		}
		string[m] = permutation[string[m]];
	}

	//switching
	for ( m=0; m<6; m++ ) {
		if( string[m*2] > string[m*2+1] ) {
			swap( &string[m*2], &string[m*2+1], sizeof(int) );
		}
	}
	
	//sorting around first central vertex
	int n;
	for ( n = 0; n<2; n++ ) {
		for ( m=0; m<2; m++ ) {
			int num1 = string[m*2]*10+string[m*2+1]; 
			int num2 = string[m*2+2]*10+string[m*2+3];
			
			if (num1 > num2) {
				swap( &string[m*2], &string[m*2+2], 2 * sizeof(int) );
			}
		}
	}
	
	//sorting around second central vertex
	for ( n = 0; n<2; n++ ) {
		for ( m=3; m<5; m++ ) {
			int num1 = string[m*2]*10+string[m*2+1]; 
			int num2 = string[m*2+2]*10+string[m*2+3];
			
			if (num1 > num2) {
				swap( &string[m*2], &string[m*2+2], 2 * sizeof(int) );
			}
		}
	}
}



int is_cycle( graph_t *mgraph ) {
		//compute number of edges
	int num_edges = 0, j, k;
	for (j = 0; j<4; j++) {
		for (k = j; k<5; k++) {
			if (g_is_edge( mgraph, j, k ) ) {
				num_edges++;
			}
		}
	}
				
	if( num_edges == 4 ) {
		int num_components = g_num_components( mgraph );
		
		if ( num_components == 2 ) {
			return 1;
		} else if( num_components == 1 ) {
			return 0;
		} else {
			msg( LOG_ERR, "impossible number of components. Something bad happened" );
			exit(1);
		}
	} else if (num_edges == 3) {
		//if 3 vertices have degree 2 then there is a triangle
		int num_deg2 = 0, num_deg0 = 0;
		
		for (j = 0; j<5; j++) {
			int current_deg = 0;
			for (k = 0; k<5; k++) {
				//no loops possible
				if (j==k) continue;
				
				if (g_is_edge( mgraph, j, k ) ) {
					current_deg++;
				}
				if (current_deg==2) num_deg2++;
				else if (current_deg==0) num_deg0++;
				else {
					//some vertex has other degree than 0 or 2, meaning there
					//is no cycle	
					return 0;
				}
			}
		}
		
		if (num_deg2 == 3 && num_deg0 == 2 ) {
			return 1;
		}
	}
	
	return 0;
}



int main(int argc, char *argv[])
{

	
	
	graph_t *graph = g_create( 26 );

	//create graph and enumerate edges so that edges neighbouring with central
	//vertices are the last

	int i, j, num_edges = 0;
	for (i = 0; i<6; i++ ) {
		for (j = 0; j<3; j++ ) {
			int vertex1 = i * 4 + 2;
			int vertex2 = vertex1 + j + 1;
			g_add_edge( graph, vertex1, vertex2 );
			g_set_data_both_directions( graph, num_edges++, vertex1, vertex2 );
		}
	}

	g_add_edge( graph, 0, 2 );
	g_add_edge( graph, 0, 6 );
	g_add_edge( graph, 0, 10 );
	
	g_add_edge( graph, 1, 14 );
	g_add_edge( graph, 1, 18 );
	g_add_edge( graph, 1, 22 );

	g_add_edge( graph, 0, 1 );
	
	g_set_data_both_directions( graph, num_edges++, 0, 2 );
	g_set_data_both_directions( graph, num_edges++, 0, 6 );
	g_set_data_both_directions( graph, num_edges++, 0, 10 );

	g_set_data_both_directions( graph, num_edges++, 1, 14 );
	g_set_data_both_directions( graph, num_edges++, 1, 18 );
	g_set_data_both_directions( graph, num_edges++, 1, 22 );

	g_set_data_both_directions( graph, num_edges++, 0, 1 );
	
	//create linegraph
	graph_t *linegraph = g_create_linegraph_from_enumerated_graph( graph, num_edges );
	
	graph_t *graph_of_colorings;	

	coloring_t *coloring = g_create_coloring( 5, linegraph->num_vertices );
	
	code_properties_t code_properties = g_code_properties( 5, linegraph->num_vertices ); //num_colors = 5

	
	int num_colorings=0, number_of_non_kempe_equivalent = 0;

	char **dictionary;
	
	int indexes[6] = {0, 0, 0, 0, 0, 0};
	
	
	//for every configuration
	for ( i = 955469; i < 1000000; i++ ) {
		//set current indexes
		int val = i;
		for (j = 5; j>=0; j-- ) {
			indexes[j] = val % 10;
			val = val/10;
		}
		
		set_prefix( coloring, 6, num_edges, indexes );

		//the coloring is correct when the colors of edges around central vertex are 0,1,2,3
		int correct_coloring = 0;

		num_colorings = 0;
		while( g_get_next_coloring_offset( linegraph, coloring, num_edges-7 ) ) {
			if ( ( coloring->colors[num_edges-7] == 0 ) &&
				( coloring->colors[num_edges-6] == 1 ) &&
				( coloring->colors[num_edges-5] == 2 ) &&
				( coloring->colors[num_edges-4] == 3 ) &&
				( coloring->colors[num_edges-3] == 0 ) &&
				( coloring->colors[num_edges-2] == 1 ) ) {
				correct_coloring = 1;
			}
			num_colorings++;
		}

		if ( correct_coloring == 0 ) continue;
		
		set_prefix( coloring, 6, num_edges, indexes );
		
		dictionary = g_create_dictionary( num_colorings );
		graph_of_colorings = g_create( num_colorings );

		num_colorings=0;
		while ( g_get_next_coloring_offset( linegraph, coloring, num_edges-7 ) ) {
			dictionary[num_colorings] = g_create_code( &code_properties );
			g_coloring_to_code( coloring, dictionary[num_colorings], &code_properties );
			num_colorings++;
		}
	
		int m;
		//perform switching
		for (m=0; m<num_colorings; m++) {
			g_code_to_coloring( dictionary[m], coloring, &code_properties );
			perform_kempe_switching_on_coloring( linegraph, graph_of_colorings, dictionary, coloring, num_edges-7 );
		}
		
		nv_t nc = g_num_components( graph_of_colorings );

		//now we have all colorings in dictionary and in graph_of_colorings
		//and we also know how many components there are in graph of colorings
	
		//so we have to find out whether there is a coloring, such that it's
		//mg contains a cycle.
		
		int cycle = 0;
		
		for (m = 0; m<num_colorings; m++ ) {
			g_code_to_coloring( dictionary[m], coloring, &code_properties );			
			
			{
				graph_t *mgraph = g_create( 5 );
				g_add_edge( mgraph, complement[indexes[0]][0], complement[indexes[0]][1] );
				g_add_edge( mgraph, complement[indexes[1]][0], complement[indexes[1]][1] );
				g_add_edge( mgraph, complement[indexes[2]][0], complement[indexes[2]][1] );

				//find out what the last (fourth) edge is
				int j, used[5], vertex1=-1, vertex2=-1;
				for (j = 0; j<5; j++)
					used[j]=0;
				
				for (j = 21; j<24; j++)
					used[coloring->colors[j]]=1;
				
				for (j = 0; j<5; j++) {
					if (used[j]==0) {
						if (vertex1<0) vertex1=j;
						else vertex2=j;
					}
				}
				g_add_edge( mgraph, vertex1, vertex2 );
				
				cycle = is_cycle( mgraph );
				g_destroy( &mgraph );
			}
			if (cycle) break;
			
			{
				graph_t *mgraph = g_create( 5 );
				g_add_edge( mgraph, complement[indexes[3]][0], complement[indexes[3]][1] );
				g_add_edge( mgraph, complement[indexes[4]][0], complement[indexes[4]][1] );
				g_add_edge( mgraph, complement[indexes[5]][0], complement[indexes[5]][1] );
				
				//find out what the last (fourth) edge is
				int j, used[5], vertex1=-1, vertex2=-1;
				for (j = 0; j<5; j++)
					used[j]=0;
				
				for (j = 18; j<21; j++)
					used[coloring->colors[j]]=1;
				
				for (j = 0; j<5; j++) {
					if (used[j]==0) {
						if (vertex1<0) vertex1=j;
						else vertex2=j;
					}
				}
				g_add_edge( mgraph, vertex1, vertex2 );
				
				cycle = is_cycle( mgraph );
				g_destroy( &mgraph );
			}
			
			if (cycle) break;
		}
		
		
		if ((nc>1 && cycle==0) || (nc==1 && cycle ==1) ) {

			printf( "%d\t%lu\t%d\n", i, nc, cycle );
		}
		
		
		//destroy dictionary and goc - avoid memory leak
		for (m=0; m<num_colorings; m++) {
			free(dictionary[m]);
		}
		g_destroy_dictionary( &dictionary );
		
		g_destroy( &graph_of_colorings );
		
		//and let's go to another coloring prefix		

	}
	
		
	printf("number_of_non_kempe_equivalent = %d\n", number_of_non_kempe_equivalent );

	g_destroy( &graph );
	g_destroy( &linegraph );
	g_destroy_coloring( &coloring );

	
	
	
	
	return 0;

}
