#include "graph_lib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>

#define LOG_DEBUG 1
#define LOG_WARN 2
#define LOG_ERR 3
#define LOG_LEVEL LOG_WARN



void g_dump_goc_graphml_sub( graph_t *graph,
						graph_t *graph_of_colorings,
						graph_t *linegraph, 
						char **dictionary,
						int num_colors,
						graph_t *graph2,
						graph_t *linegraph2,
						char **dictionary2,
						int num_colorings2) {

	//print header
	printf( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" );
	printf( "<graphml xmlns=\"http://graphml.graphdrawing.org/xmlns\"\n" );
	printf( "         xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n" );  
	printf( "         xsi:schemaLocation=\"http://graphml.graphdrawing.org/xmlns\n" );  
	printf( "         http://www.yworks.com/xml/schema/graphml/1.1/ygraphml.xsd\"\n" );  
	printf( "         xmlns:y=\"http://www.yworks.com/xml/graphml\">\n" );  
	printf( "  <key id=\"d0\" for=\"node\" yfiles.type=\"nodegraphics\"/>  \n" );
	printf( "  <key id=\"d1\" for=\"edge\" yfiles.type=\"edgegraphics\"/>  \n" );
	printf( "  <graph id=\"G\" edgedefault=\"undirected\"> \n" );

	
	code_properties_t properties = g_code_properties( num_colors, linegraph->num_vertices );
	coloring_t *coloring = g_create_coloring( num_colors, linegraph->num_vertices);


	code_properties_t properties2 = g_code_properties( num_colors, linegraph2->num_vertices );
	coloring_t *coloring2 = g_create_coloring( num_colors, linegraph2->num_vertices  );

	
	//print nodes
	
	int equal_coloring;
	nv_t i;
	for ( i=0; i<graph_of_colorings->num_vertices; i++ ) {
		g_erase_coloring( coloring );
		g_code_to_coloring( dictionary[i], coloring, &properties );

		{
			nv_t i, j, k;
			for ( i = 0; i<num_colorings2; i++ ) {
				g_code_to_coloring( dictionary2[i], coloring2, &properties2 );
				equal_coloring = 1;
				
				for ( j = 0; j<graph2->num_vertices-1; j++ ) {
					for ( k = j+1; k<graph2->num_vertices; k++ ) {
						if (g_is_edge( graph, j, k) ) {
							fflush( stdout );
							if ( g_get_edge_color( graph, coloring, j, k ) !=
								g_get_edge_color( graph2, coloring2, j, k ) ) {
								equal_coloring = 0;
								break;
							}
						}
//						printf(" %lu: %d - %d \n", j, coloring2->colors[j], coloring->colors[j]);
					}
				}
				if (equal_coloring) break;
			}
		}


		printf("    <node id=\"n%lu\" yfiles.foldertype=\"folder\">\n", i );
		printf("      <data key=\"d0\">                                   \n" );
		printf("        <y:GroupNode>                                   \n" );
		printf("          <y:NodeLabel>%lu</y:NodeLabel>\n \n", i );
		
		if (equal_coloring)
			printf("          <y:Fill color=\"#ccffff\" transparent=\"false\"/> \n" );
		else
			printf("          <y:Fill color=\"#eeeeee\" transparent=\"false\"/> \n" );
			
		printf("        </y:GroupNode>                                  \n" );
		printf("      </data>                                           \n" );
		
		
		//print a colored graph
		g_dump_graphml_edgecolors( graph, coloring, i );
		printf("    </node>\n" );
	}

	

	//print edges
	struct edge_t edge;
	for ( i=0; i<graph_of_colorings->num_vertices; i++ ) {
		edge = graph_of_colorings->vertices[i];
		if ( edge.vertex<0 ) continue;
		
		while( 1 ) {
			//print edge [a,b] only if a<b, so that there are not duplicate edges
			if (i<edge.vertex) {
				printf("    <edge id=\"e%lu:%lu\" source=\"n%lu\" target=\"n%lu\">\n", i, edge.vertex, i, edge.vertex);
				printf("      <data key=\"d1\">                                            \n");
				printf("    	<y:PolyLineEdge>                                         \n");
				printf("    	  <y:LineStyle type=\"line\" width=\"2.0\" color=\"#000000\"/> \n"); 
				printf("    	  <y:EdgeLabel></y:EdgeLabel>                            \n");
				printf("    	</y:PolyLineEdge>                                        \n");
				printf("      </data>                                                    \n");
				printf("    </edge>                                                      \n");
			}
			if (edge.next==NULL) break;
			edge = *(edge.next);
		};
		
	}
	printf( "\n" );

	
	printf("  </graph>\n" );
	printf("</graphml>\n" );
	
}


FILE *fopen_and_skip_lines( char *filename, int num_lines ) {


	FILE *input_file = fopen( filename, "r" );
	if (NULL == input_file ) {
		msg (LOG_ERR, "failed to open input file for reading ");
		exit(1);	
	};
	skip_lines( num_lines, input_file );
	return input_file;
}


int main(int argc, char *argv[])
{
	int max_degree = 4, num_colors = 4;
	
	nv_t i;
	

	graph_t *graph, *linegraph, *graph_of_colorings; 

	graph_t *graph_e, *linegraph_e, *graph_of_colorings_e; 

	coloring_t *coloring, *coloring_e;
	unsigned long num_colorings = 0;
	unsigned long num_colorings_e = 0;
	char **dictionary, **dictionary_e;
	
	
	char *input_file_name = "input/vert10maxdeg4.g6";
	int num_lines = 61194;
	
	//initialize graphs (from file?)
	FILE *input_file = fopen_and_skip_lines( input_file_name, num_lines );
	graph = g_create_from_file( input_file );
	fclose( input_file );
	
	{
			int max_degree_index = 0;
			while ( g_vertex_degree( graph, max_degree_index) < max_degree ) {
				max_degree_index++;
			}
			
			if (max_degree_index != 0 ) {
				
				graph_t *tmp_graph = g_create( graph->num_vertices );
				int i, j;
				for ( i = 0; i<graph->num_vertices-1; i++ ) {
					for ( j = i+1; j<graph->num_vertices; j++ ) {
						if (g_is_edge( graph, i, j ) ) {
							int new_i, new_j;
							
							if (i == 0) new_i = max_degree_index;
							else if (i == max_degree_index) new_i = 0;
							else new_i = i;


							if (j == 0) new_j = max_degree_index;
							else if (j == max_degree_index) new_j = 0;
							else new_j = j;
							g_add_edge( tmp_graph, new_i, new_j );
						}
					}
				}
				
				g_destroy( &graph );
				graph = tmp_graph;
			}
		}
	
		g_remove_edge( graph, 3, 6 );	
		graph_e = g_duplicate( graph );
	
//	g_remove_edge( graph_e, 7, 9 );	
	
/*	
	
	
	g_remove_edge( graph, 7, 9 );	
	g_remove_edge( graph_e, 3, 6 );	
	*/	
//	g_remove_edge( graph, 3, 4 );
	
	
	
	
	linegraph = g_create_linegraph( graph );
	
	//printf(" data[3,4] = %d\n", g_get_data( graph, 3, 4 ) );
	
	coloring = g_create_coloring( num_colors, linegraph->num_vertices );		
	code_properties_t code_properties = g_code_properties( num_colors, linegraph->num_vertices );
	
	//count number of colorings
	num_colorings = 0;
	
	g_init_coloring_for_regular_graph( coloring, max_degree );
	while (g_get_next_coloring_offset( linegraph, coloring, max_degree ) ) {
		num_colorings++;
	}

//	printf("num_colorings: %lu\n", num_colorings );
	
	//create graph of colorings
	graph_of_colorings = g_create( num_colorings );

	//set coloring prefix
	g_init_coloring_for_regular_graph( coloring, max_degree );
	
	//initialize dictionary
	dictionary = g_create_dictionary( num_colorings );
	num_colorings=0;
	//store all colorings to dictionary
	while (g_get_next_coloring_offset( linegraph, coloring, max_degree ) ) {
		dictionary[num_colorings] = g_create_code( &code_properties );
		g_coloring_to_code( coloring, dictionary[num_colorings], &code_properties );
		num_colorings++;
	}	
	
	//perform switching
	for (i=0; i<num_colorings; i++) {
		g_code_to_coloring( dictionary[i], coloring, &code_properties );
		perform_kempe_switching_on_coloring( linegraph, graph_of_colorings, dictionary, coloring, max_degree );
	}
	


	
	
	
	
	linegraph_e = g_create_linegraph( graph_e );

	coloring_e = g_create_coloring( num_colors, linegraph_e->num_vertices );		
	code_properties_t code_properties_e = g_code_properties( num_colors, linegraph_e->num_vertices );
	
	//count number of colorings
	num_colorings = 0;
	
	g_init_coloring_for_regular_graph( coloring_e, max_degree );
	while (g_get_next_coloring_offset( linegraph_e, coloring_e, max_degree ) ) {
		num_colorings_e++;
	}

	
	//create graph of colorings
	graph_of_colorings_e = g_create( num_colorings_e );

	//set coloring prefix
	g_init_coloring_for_regular_graph( coloring_e, max_degree );
	
	//initialize dictionary
	dictionary_e = g_create_dictionary( num_colorings_e );
	num_colorings_e=0;
	//store all colorings to dictionary
	while (g_get_next_coloring_offset( linegraph_e, coloring_e, max_degree ) ) {
		dictionary_e[num_colorings_e] = g_create_code( &code_properties_e );
		g_coloring_to_code( coloring_e, dictionary_e[num_colorings_e], &code_properties_e );
		num_colorings_e++;
	}	
	
	//perform switching
	for (i=0; i<num_colorings_e; i++) {
		g_code_to_coloring( dictionary_e[i], coloring_e, &code_properties_e );
		perform_kempe_switching_on_coloring( linegraph_e, graph_of_colorings_e, dictionary_e, coloring_e, max_degree );
	}

	fprintf( stderr, "number of components: %lu\n", g_num_components( graph_of_colorings ) );

	
	
//	g_dump_tgf( graph_of_colorings );
//	g_dump_wilma( graph_of_colorings_e );
	
		g_dump_goc_graphml_sub( graph_e, graph_of_colorings_e, linegraph_e, dictionary_e, num_colors, graph, linegraph, dictionary, graph_of_colorings->num_vertices );
//		g_dump_wilma( graph_of_colorings_e );
//		exit(0);


	for (i=0; i<num_colorings; i++) {
		free(dictionary[i]);
	}
	g_destroy_dictionary( &dictionary );
	g_destroy_coloring( &coloring );
	g_destroy( &graph );
	g_destroy( &linegraph );
	g_destroy( &graph_of_colorings );

	for (i=0; i<num_colorings_e; i++) {
		free(dictionary_e[i]);
	}
	g_destroy_dictionary( &dictionary_e );
	g_destroy_coloring( &coloring_e );
	g_destroy( &graph_e );
	g_destroy( &linegraph_e );
	g_destroy( &graph_of_colorings_e );


	return 0;

}
