#include "graph_lib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>

#define LOG_DEBUG 1
#define LOG_WARN 2
#define LOG_ERR 3
#define LOG_LEVEL LOG_WARN


FILE *fopen_and_skip_lines( char *filename, int num_lines ) {


	FILE *input_file = fopen( filename, "r" );
	if (NULL == input_file ) {
		msg (LOG_ERR, "failed to open input file for reading ");
		exit(1);	
	};
	skip_lines( num_lines, input_file );
	return input_file;
}


int main(int argc, char *argv[])
{
	
	int number_of_graph_to_be_inspected = -1;
	
	if (argc != 3 && argc != 4 ) {
		printf( "USAGE: %s file_containing_subgraphs file_with_graphs_to_be_inspected [number_of_graph_to_be_inspected] \n", argv[0] );
		printf( "       if you enter the number of graph, only one graph will be inspected\n" );
		return 1;
	}

	if (argc == 4) {
		number_of_graph_to_be_inspected = atoi( argv[3] );
	}
	
	
	FILE *subgraphs_file = fopen( argv[1], "r" );
	if (NULL == subgraphs_file ) {
		msg ( LOG_ERR, "failed to open file containing subgraphs %s", argv[1] );
		exit( 1 );	 
	};
	
	long num_subgraphs = count_num_lines( argv[1] );

//	printf( "num_subgraphs: %ld \n", num_subgraphs );
	
	
	
	FILE *graphs_file = fopen( argv[2], "r" );
	if (NULL == graphs_file ) {
		msg ( LOG_ERR, "failed to open file containing graphs %s", argv[2] );
		exit( 1 );	
	};

	graph_t *graph, *subgraphs[num_subgraphs];

	//load subgraphs
	int number_of_subgraph = 0;
	while( 	NULL != ( subgraphs[number_of_subgraph] = g_create_from_file( subgraphs_file ) ) ) {
		number_of_subgraph++;
	}
	fclose( subgraphs_file );
	

	int graph_counter = 0;
	
	if( number_of_graph_to_be_inspected>0 ) {
		skip_lines( number_of_graph_to_be_inspected, graphs_file );
		graph_counter = number_of_graph_to_be_inspected;
	}
	
	number_of_graph_to_be_inspected = -1;
	
	//check every graph in the graph_file
	while( 	NULL != ( graph = g_create_from_file( graphs_file ) ) ) {
		printf("graf cislo: %d \t", graph_counter++ );
		int bipartiteness = g_bipartiteness( graph );
		printf("%i\t", bipartiteness );

		if ( bipartiteness == 0 ) {
			printf("\n");
			g_destroy( &graph );
			if( number_of_graph_to_be_inspected>=0 ) {
				break;
			}
			continue;
		}
		
		
		int found = 0;
		//compare graph with every subgraph
		for ( number_of_subgraph = 0; number_of_subgraph < num_subgraphs; number_of_subgraph++ ) {
			fprintf(stderr, "number_of_subgraph: %d\n", number_of_subgraph);
			
			
			
			
			//assign this so I don't have to use the index all the time
			graph_t *subgraph = subgraphs[number_of_subgraph];
			int i;
			
			
			//subgraf can't have more vertices than graph
			if (subgraph->num_vertices > graph->num_vertices) {
				continue;
			}
			
			//subgraf can't have more edges than graph
			if ( g_enumerate_edges( subgraph ) > g_enumerate_edges( graph ) ) {
				continue;
			}

			short graph_degrees[graph->num_vertices];
			short subgraph_degrees[subgraph->num_vertices];

			
			//we have to find a position of each subgraph's vertex in graph
			//initialize the positions to -1
			int positions[subgraph->num_vertices];
			for ( i = 0; i < subgraph->num_vertices; i++ ) {
				subgraph_degrees[i] = g_vertex_degree( subgraph, i );
				positions[i] = -1;
			}
			
			//each position can be used only once
			//set every position to "not used"
			int used[graph->num_vertices];
			for ( i = 0; i < graph->num_vertices; i++ ) {
				graph_degrees[i] = g_vertex_degree( graph, i );
				used[i] = 0;
			}
			
			
			
			
			
			
			
			//now for the backtrack itself
			int current_vertex = 0;
			while ( current_vertex >= 0 && current_vertex<subgraph->num_vertices ) {
				
				
				
				//now set new position of current_vertex
				if( positions[current_vertex] > -1 ) {
					used[positions[current_vertex]] = 0;
				}
				
				positions[current_vertex]++;
				while( used[positions[current_vertex]] && positions[current_vertex] < graph->num_vertices ) {
					positions[current_vertex]++;
				}
				
				//now it either found good position or it goes off grid
				if ( positions[current_vertex] < graph->num_vertices ) {
					used[positions[current_vertex]] = 1;
				} else {
					positions[current_vertex]=-1;
					current_vertex--;
					continue;
				}

				//check, whether current_vertex is ok.
				int vertex_is_ok = 1;
				
				
				if ( subgraph_degrees[current_vertex] > graph_degrees[positions[current_vertex]] ) {
					vertex_is_ok = 0;
				} else {
					for ( i = 0; i<current_vertex; i++ ) {
						if (g_is_edge( subgraph, i, current_vertex ) ) {
							if (!g_is_edge( graph, positions[i], positions[current_vertex] ) ) {
								vertex_is_ok = 0;
								break;
							}
						}
					}
				}
				
				//If it is, increase current_vertex counter and go to next cycle
				if( vertex_is_ok ) {
					current_vertex++;
					continue;
				}

				//if it isn't just go to the next cycle with the same current vertex
				else {
					continue;
				}		
				
			}
			
			if ( current_vertex > 0 ) {
				printf( "sub%d\n", number_of_subgraph );
				found = 1;
				break;
			}
		}
		if (found) {}//printf("\n");
		else {
			FILE *subgraphs_file = fopen( argv[1], "a" );
			if (NULL == subgraphs_file ) {
				msg ( LOG_ERR, "failed to open file containing subgraphs for writing: %s", argv[1] );
				exit( 1 );	 
			};
	
			printf( "orig%lu\t", num_subgraphs );
			g_write_to_file( graph, stdout );
			//g_write_to_file( graph, subgraphs_file );
			fclose( subgraphs_file );			
			fflush( stdout );
		}


		if( number_of_graph_to_be_inspected>=0 ) {
			break;
		}

	}

	return 0;

}
