//#include <stdio.h>
#include "graph_lib.h"
#include <stdlib.h>

void g_dump_coloring_from_to( coloring_t *coloring, nv_t from, nv_t to ) {
	nv_t i;
	if (to > coloring->num_vertices) {
		msg( LOG_ERR, "to > coloring->num_vertices" );
		exit(0);
	}
	if (from >= coloring->num_vertices) {
		msg( LOG_ERR, "from >= coloring->num_vertices" );
		exit(0);
	}
	
	
	for ( i=0; i<to-from; i++ ) {
		printf( "%2d ", coloring->colors[i+from] );
		if ((i+1)%50==0) printf("\n");
	}
	printf( "\n" );
}


nv_t g_get_length_of_kempe_path( coloring_t *coloring, 
					graph_t *graph, nv_t vertex, color_t new_color )
{
	
	color_t color1 = new_color, color2 = coloring->colors[vertex];
//	printf( "initial_vertex: %d,   color1 = %d    color2 = %d \n", vertex, color1, color2 );
	
	nv_t queue[coloring->num_vertices];
	nv_t mark[coloring->num_vertices];
	nv_t i, position, num_queued=0, current_vertex;
	edge_t *edge;
	edge_iterator_t iterator;
	
	for( i=0; i<coloring->num_vertices; i++ ) {
		queue[i] = -1;
		mark[i] = 0;
	}


	//put first vertex to the queue
	position=0;
	num_queued=0;
	queue[position+num_queued] = vertex;
	num_queued++;
	
	while ( num_queued>0 ) {
		
		current_vertex = queue[position]; //pop vertex from queue
		num_queued--;
		position++;
		
		mark[current_vertex] = 1; //mark it for change
		
		iterator = g_new_edge_iterator(graph); //get all neighbors of current vertex
		iterator.vertex = current_vertex;
	
		//add to queue - each neighbor not yet marked
		while ( NULL!= (edge = g_edge_iterator_next_for_vertex( &iterator ) ) ) {
			if (mark[edge->vertex]==0 && ( coloring->colors[edge->vertex]==color1 || coloring->colors[edge->vertex]==color2 ) ) { 
				//add vertex to queue
				queue[position+num_queued] = edge->vertex;
				num_queued++;
			} else {
				//vertex is already marked for switch or wrong color
				// so we don't do anything 
			}
		}
	}
	
	nv_t length = 0;
	
	for ( i=0; i<coloring->num_vertices; i++) {
		if (mark[i]==1) {
			length++;
		}
	}
	
	return length;
}


char color_names[][10] = { "red", "green", "blue", "yellow", "black", "cyan", "grey", "violet", "lightblue", "maroon" };







int main(int argc, char *argv[]) {	
	
	//zacnem s 2 grafmi K_n
	//postupne je: 
	//   n nad 2 hran Kn
	//   dtto v druhom grafe
	//   n priamych spajajucich hran
	//   n*(n-1) sikmych spajajucich hran
	// dostanem K_(2n+2) = 2n nad 2
	
	if ( argc != 2 ) {
		printf(" USAGE:   <filneame>   <number of vertices of complete graph> \n");
		printf("                       has to be even number \n");
		printf(" e.g. %s 8 \n", argv[0]);
		return 0;
	}

	int n = atoi(argv[1]);

	graph_t *orig_graph = g_create( n );

	int i, j;

	for ( i=0; i<n-1; i++ ) {
		for ( j=i+1; j<n; j++ ) {
			g_add_edge( orig_graph, i, j );
		}
	}

	//and vertices n..2n-1 induce second one
	
	
	graph_t *graph = g_create_linegraph( orig_graph );
	
	g_dump( orig_graph );
	g_dump( graph );
		
	coloring_t *coloring = g_create_coloring( n-1, graph->num_vertices ); //n-1 colors

	int counter =0;	

	int ok;
	while (1 == g_get_next_coloring_offset( graph, coloring, 0 ) ) {
		ok=1;
		
		if ( coloring->colors[n-2] != n-2 ) {
			printf("nothing more found\n");
			exit( 0 );
		}
		
		//this checks lengths of 1-factors. Tries to find longest alterning paths starting in every edge of K_n
		//and every color from the set of added edges (indirectly connecting ones)
		for (i = 0; i<1; i++ ) { //vertices
			for (j=0; j<coloring->num_colors; j++) { //colors
				
				if ( coloring->colors[i]==j ) {
					continue;
				}
				
				if (g_get_length_of_kempe_path( coloring, graph, i, j ) != n ) {
					ok = 0;
					break;
				}
			}
			if (ok==0) break;
		}
		
		
		if (ok) {
			printf("coloring found: " );
			
			printf( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" );
			printf( "<graphml xmlns=\"http://graphml.graphdrawing.org/xmlns\"\n" );
			printf( "         xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n" );  
			printf( "         xsi:schemaLocation=\"http://graphml.graphdrawing.org/xmlns\n" );  
			printf( "         http://www.yworks.com/xml/schema/graphml/1.1/ygraphml.xsd\"\n" );  
			printf( "         xmlns:y=\"http://www.yworks.com/xml/graphml\">\n" );  
			printf( "  <key id=\"d0\" for=\"node\" yfiles.type=\"nodegraphics\"/>  \n" );
			printf( "  <key id=\"d1\" for=\"edge\" yfiles.type=\"edgegraphics\"/>  \n" );
					
			g_dump_graphml_edgecolors( orig_graph, coloring, 0 );

			printf("</graphml>");
			counter++;
			if (counter==2) exit(0);
			
//			g_dump_coloring( coloring );
//			exit(0);
		}
		
		
	}

	
	printf("nothing more found\n");
	
	g_destroy( &orig_graph );
	g_destroy( &graph );


	
	return 0;
}
