#include "graph_lib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>

#define LOG_DEBUG 1
#define LOG_WARN 2
#define LOG_ERR 3
#define LOG_LEVEL LOG_WARN


//get only filename (without path) from argument for input file
char* get_only_filename( char* filename_with_path ) {	
	int i=0;
	while (filename_with_path[i]!='\0') i++;
	while (filename_with_path[i]!='/' && i>0) i--;
	if (i>0) i++;
	return &filename_with_path[i];
}



//ugly function. Assumes vertex to be of degree 2
//graph properly colored by 3-edge-coloring
int missing_color( graph_t* graph, coloring_t *coloring, int vertex_number ) {
	int used[3] = {0, 0, 0};
	edge_t *edge = &graph->vertices[vertex_number];
	used[coloring->colors[edge->data]] = 1;
	edge = edge->next;
	used[coloring->colors[edge->data]] = 1;
	
	int i=0;
	while (used[i]!=0) i++;
	return i;
}

//ugly function.
int is_marked( graph_t *graph, int vertex_number, nv_t *mark ) {
	edge_t *edge = &graph->vertices[vertex_number];
	return mark[ edge->data ] == 1 || mark[ edge->next->data ] == 1;
}




int main(int argc, char *argv[])
{
	int max_degree, num_colors; 
	
	if (argc != 3 && argc != 4) {
		fprintf( stderr, "USAGE: %s input_file number_of_colors [number_of_a_graph]\n", argv[0] );
		fprintf( stderr, "\tit is assumed that the graph has a vertex of degree D, \n\twhere number_of_colors-1 <= D <= number_of_colors\n" );
		fprintf( stderr, "\n\tif number_of_a_graph is given, only one graph is processed\n" );
		return 0;
	} else {
		num_colors = atoi( argv[2] );
		if (num_colors!=3) {
			fprintf( stderr, "       num colors must be 3. sry.\n" );
			return 0;
		}
	}

	fprintf( stderr, "opening %s\n", argv[1] );
	FILE *input_file = fopen( argv[1], "r" );
	if (NULL == input_file ) {
		msg (LOG_ERR, "failed to open input file for reading ");
		exit(1);	
	}; 
	
	
	
	char *filename = get_only_filename( argv[1] );
		
	
	char output_meta_file_name[60];
	sprintf( output_meta_file_name, "output/segment_col%i_%s", num_colors, filename );
	
	long graph_offset = 0;
	int is_num_graph_given;
	if( argc == 4 ) {
		is_num_graph_given = 1;
		graph_offset = atoi( argv[3] );
	} else {
		is_num_graph_given = 0;
		graph_offset = count_num_lines( output_meta_file_name );
	}
	
		
	skip_lines( graph_offset, input_file );

	//create empty meta file if it doesn't exist
	if ( is_num_graph_given ) {
			fprintf( stdout, "filename\tgraph#\tnum_vertices\tnum_edges\tvert0\tvert1\tvert2\tbipartiteness\tis_segment\tnote\n" );
	} if ( !is_num_graph_given ) {
		FILE *output_meta_file = fopen( output_meta_file_name, "a" );
		//if meta file doesn't contain any graph results, then print header to meta file	
		if ( graph_offset == 0 ) {
			fprintf( output_meta_file, "filename\tgraph#\tnum_vertices\tnum_edges\tvert0\tvert1\tvert2\tbipartiteness\tis_segment\tnote\n" );
		}

		fclose( output_meta_file );
	}


	graph_t *graph, *linegraph; 
	coloring_t *coloring;
	code_properties_t code_properties;
	unsigned long num_colorings = 0;
	
	long count_graphs = graph_offset;

	fprintf( stderr, "is_num_graph_given : %d\n", is_num_graph_given );
	
	
	//loop for all file lines (i.e. all graphs
	while( 	NULL != ( graph = g_create_from_file( input_file ) ) ) { 	
		//if specific graph number is given, then only one graph should be
		//processed. If loop wants to process other graph, break the loop
		if ( is_num_graph_given && count_graphs != graph_offset ) {
			break;
		}
		
		//if graph doesn't contain vertex of degree max_degree, then skip it
		//else convert it, so that vertex 0 has "max_degree" neighbours
		max_degree = g_max_degree( graph );
		
		int num_maxdeg2_vertices = num_vertices_with_maxdegree( graph, 2 );
		//or max_degree>num_colors (can't be colored)
		//or max_degree<num_colors-1 (is kempe-equivalent for sure)
		//go to the next graph
		if ( max_degree < num_colors-1 ||
			 num_colors < max_degree ||
			 num_maxdeg2_vertices < 2
		) {
			g_destroy( &graph );

			count_graphs++;
			continue;
		}
				
		
		max_degree = g_max_degree( graph );
		
		int deg2_vertices[num_maxdeg2_vertices]; //array of indexes of degree2 vertices
		int i, j = 0;
		for ( i = 0; i < graph->num_vertices; i++ ) {
			if ( g_vertex_degree( graph, i ) <= 2 ) {
				deg2_vertices[j] = i;
				j++;
			}
		}
		

		int index_of_vertex0, index_of_vertex1, index_of_vertex2;

		for (index_of_vertex0 = 0; index_of_vertex0 < num_maxdeg2_vertices-2; index_of_vertex0++ )	{
			for (index_of_vertex1 = index_of_vertex0+1; index_of_vertex1 < num_maxdeg2_vertices-1; index_of_vertex1++ )	{
				for (index_of_vertex2 = index_of_vertex1+1; index_of_vertex2 < num_maxdeg2_vertices; index_of_vertex2++ )	{
					nv_t i, j;
									
					int vert0 = deg2_vertices[index_of_vertex0];
					int vert1 = deg2_vertices[index_of_vertex1];
					int vert2 = deg2_vertices[index_of_vertex2];

					
					graph_t *extended_graph = g_duplicate_and_extend( graph, 3 );
					g_add_edge( extended_graph, vert0, extended_graph->num_vertices-3 );
					g_add_edge( extended_graph, vert1, extended_graph->num_vertices-2 );
					g_add_edge( extended_graph, vert2, extended_graph->num_vertices-1 );

					int num_edges = 0;
					g_set_data_everywhere( extended_graph, -1 );

					
					
					g_set_data_both_directions( extended_graph, num_edges++, vert0, extended_graph->num_vertices-3 );
					g_set_data_both_directions( extended_graph, num_edges++, vert1, extended_graph->num_vertices-2 );
					g_set_data_both_directions( extended_graph, num_edges++, vert2, extended_graph->num_vertices-1 );
										
					
					
					for ( i = 0; i < extended_graph->num_vertices-1; i++ ) {
						for ( j = i+1; j < extended_graph->num_vertices; j++ ) {
							if( g_is_edge( extended_graph, i, j ) ) {
								if ( g_get_data( extended_graph, i, j ) < 0 ) {
									g_set_data_both_directions( extended_graph, num_edges++, i, j );
								}
							}
						}
					}

					linegraph = g_create_linegraph_from_enumerated_graph( extended_graph, num_edges );
						
					coloring = g_create_coloring( num_colors, linegraph->num_vertices );		
					
					code_properties = g_code_properties( num_colors, linegraph->num_vertices );
					
					//count number of colorings
					num_colorings = 0;
							
					nv_t coloring_offset = 1; //coloring offset means how many edges
					//should have fixed coloring. offset, because we color only
					//the other edges
					
					g_init_coloring_for_regular_graph( coloring, coloring_offset );
					
					int tmp;
					int wrong_coloring = 0, one_component = 1;

					while ( ( tmp = g_get_next_coloring_offset( linegraph, coloring, coloring_offset ) ) > 0 ) {
						num_colorings++;
						if ( coloring->colors[0] == coloring->colors[1] || coloring->colors[0] == coloring->colors[2] ) {
							wrong_coloring = 1;	
							break;
						}

						//a ci po prepnuti hran pri vert0 na farbu 2 ma vert2 opat spravne farby hran
						nv_t mark[coloring->num_vertices];

						//test edge0 switch to color of edge1
						g_breadth_first_search( linegraph, coloring, 0, coloring->colors[1], mark );
						if( !mark[1] ) {
							one_component = 0;
							break;
						}
	
						//test edge0 switch to color of edge2
						g_breadth_first_search( linegraph, coloring, 0, coloring->colors[2], mark );
						if( !mark[2] ) {
							one_component = 0;
							break;
						}

						//test edge1 switch to color of edge2
						g_breadth_first_search( linegraph, coloring, 1, coloring->colors[2], mark );
						if( !mark[2] ) {
							one_component = 0;
							break;
						}
						
						if (tmp==2) break;			
					}
	
					FILE *output, *output_meta_file;
					//output results
					if ( is_num_graph_given ) {
						output=stdout;
					} else {
						output_meta_file = fopen( output_meta_file_name, "a" );
						if (NULL == output_meta_file ) {
							msg (LOG_ERR, "failed to open metafile for writing ");
							exit(1);	
						};
						output = output_meta_file;
					}
					
	
					if ( num_colorings < 1 ) {
					//		fprintf( output, "0\tnot colorable\n" );
					} else if ( !one_component ) {
					//		fprintf( output, "0\tnot one kempe component\n" );
					} else if ( wrong_coloring ) {
					//		fprintf( output, "0\twrong coloring is possible\n" );
					} else {
						fprintf( output, "%s\t%lu\t%i\t%i\t%i\t%i\t%i\t%i\t1\tvertex\n", filename, count_graphs, graph->num_vertices, linegraph->num_vertices-3, vert0, vert1, vert2, g_bipartiteness( graph ) );
					}
						
					if ( !is_num_graph_given ) {
						fclose( output_meta_file );
					}
				
					g_destroy_coloring( &coloring );
					g_destroy( &linegraph );
					g_destroy( &extended_graph );
			
		
				}
			}
		}

		
		
		
		
		
		
		
		
		
		for (index_of_vertex0 = 0; index_of_vertex0 < num_maxdeg2_vertices-1; index_of_vertex0++ )	{
			for (index_of_vertex1 = index_of_vertex0+1; index_of_vertex1 < num_maxdeg2_vertices; index_of_vertex1++ )	{
					nv_t i, j;
									
					int vert0 = deg2_vertices[index_of_vertex0];
					int vert1 = deg2_vertices[index_of_vertex1];

					
					graph_t *extended_graph = g_duplicate_and_extend( graph, 2 );
					g_add_edge( extended_graph, vert0, extended_graph->num_vertices-2 );
					g_add_edge( extended_graph, vert1, extended_graph->num_vertices-1 );

					int num_edges = 0;
					g_set_data_everywhere( extended_graph, -1 );

					
					g_set_data_both_directions( extended_graph, num_edges++, vert0, extended_graph->num_vertices-2 );
					g_set_data_both_directions( extended_graph, num_edges++, vert1, extended_graph->num_vertices-1 );
										
					
					for ( i = 0; i < extended_graph->num_vertices-1; i++ ) {
						for ( j = i+1; j < extended_graph->num_vertices; j++ ) {
							if( g_is_edge( extended_graph, i, j ) ) {
								if ( g_get_data( extended_graph, i, j ) < 0 ) {
									g_set_data_both_directions( extended_graph, num_edges++, i, j );
								}
							}
						}
					}

					
					linegraph = g_create_linegraph_from_enumerated_graph( extended_graph, num_edges );
						
					coloring = g_create_coloring( num_colors, linegraph->num_vertices );		
					
					code_properties = g_code_properties( num_colors, linegraph->num_vertices );
					
					//count number of colorings
					num_colorings = 0;
							
					nv_t coloring_offset = 1; //coloring offset means how many edges
					//should have fixed coloring. offset, because we color only
					//the other edges
					
					g_init_coloring_for_regular_graph( coloring, coloring_offset );
					
					int tmp;
					int vertex_block = 1, edge_block = 1, one_component = 1;

					while ( ( tmp = g_get_next_coloring_offset( linegraph, coloring, coloring_offset ) ) > 0 ) {
						num_colorings++;

						//if the colors of edges are the same, it can't be vertex_block anymore
						if ( vertex_block && coloring->colors[0] == coloring->colors[1] ) {
							vertex_block = 0;	
						}

						//if the colors of edges are different, it can't be edge_block anymore
						if ( edge_block && coloring->colors[0] != coloring->colors[1] ) {
							edge_block = 0;	
						}

						//if it is not edge_block nor vertex_block, exit the loop
						if ( !edge_block && !vertex_block ) {
							break;
						}
						
						//a ci po prepnuti hran pri vert0 na farbu 2 ma vert2 opat spravne farby hran
						nv_t mark[coloring->num_vertices];

						//now it centairly can't be both, vertex_block and edge_block
						//so if it's edge_block, both color changes on first connector
						//must be done also on second connector
						if (edge_block) {
							g_breadth_first_search( linegraph, coloring, 0, (coloring->colors[0]+1)%3, mark );
							if( !mark[1] ) {
								one_component = 0;
								break;
							}
							g_breadth_first_search( linegraph, coloring, 0, (coloring->colors[0]+2)%3, mark );
							if( !mark[1] ) {
								one_component = 0;
								break;
							}
							
						//else if it's vertex_block and you switch color of first connector
						//to color of second connector, the second connector must also switch
						} else if (vertex_block) {
							g_breadth_first_search( linegraph, coloring, 0, coloring->colors[1], mark );
							if( !mark[1] ) {
								one_component = 0;
								break;
							}
						}
						if (tmp==2) break;			
					}
	
					FILE *output, *output_meta_file;
					//output results
					if ( is_num_graph_given ) {
						output=stdout;
					} else {
						output_meta_file = fopen( output_meta_file_name, "a" );
						if (NULL == output_meta_file ) {
							msg (LOG_ERR, "failed to open metafile for writing ");
							exit(1);	
						};
						output = output_meta_file;
					}
					
	
					if ( num_colorings < 1 ) {
					//		fprintf( output, "0\tnot colorable\n" );
					} else if ( !one_component ) {
					//		fprintf( output, "0\tnot one kempe component\n" );
					} else if ( !vertex_block && !edge_block ) {
					//		fprintf( output, "0\twrong coloring is possible\n" );
					} else {
						fprintf( output, "%s\t%lu\t%i\t%i\t%i\t%i\t%i\t%i\t1\t", filename, count_graphs, graph->num_vertices, linegraph->num_vertices-2, vert0, vert1, -1, g_bipartiteness( graph ) );
						if (vertex_block) {
							fprintf( output, "vertex\n" );
						} else if (edge_block ) {
							fprintf( output, "edge\n" );
						}
					}
						
					if ( !is_num_graph_given ) {
						fclose( output_meta_file );
					}
				
					g_destroy_coloring( &coloring );
					g_destroy( &linegraph );
					g_destroy( &extended_graph );
			
		
			
			}
		}		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		g_destroy( &graph );
		count_graphs++;
		
	
	}
	
	fclose( input_file );
	fprintf( stderr, "done\n\n\n");
	
	return 0;

}
